function [ data ] = psv_edit_chirp( data )
    if nargin ~= 1; return; end;

%
% if structure ps does not carry beam data return 
if(isfield(data.ps(data.my_axes),'b_stat'))
else
    uiwait(msgbox('No phase space data!','PSV_SUM_PLOT','error'))
    return
end
%
% speed of light
c = 3e8;
% unpack ------------------------------------------------------------------
x  = data.ps(data.my_axes).b_stat.x;
s = x(:,5);
e = x(:,6);
%
gammag = data.ps(data.my_axes).gamma;
e0 = gammag*0.511;
%
% rf frequency
nu = data.psv_rf.nu;
% harmonic number
n  = data.psv_rf.n;
%
% plot and analyze longitudinal phase space -------------------------------
[p,f_error] = polyfit(s,e,3);
%
% normalized energy and relativ momentum deviation derivatives at s=0:
n_der = [1;p(3);2*p(2);6*p(1)]; 
%
% calculate normalized amps and phases from energy and derivatives
[a1 a2 phi1 phi2] =  knobs_to_amp_phase(e0,nu,n,n_der);
%
% plot initial and fitted phase space
plot_long_ps(nu,n,s,e,p,a1,a2,phi1,phi2)
%
% store difference between fitted polynominal and energy distribution
de = e - polyval(p,s);
%
% prompt for new parameters -----------------------------------------------
prompt = {
    'Longitudinal phase space from RF settings or knobs? (0/1): ';...
    'RF fundamental mode frequency [GHz]: ';...
    'RF fundamental mode amplitude [MV]: ';...
    'RF fundamental mode phase [degrees off crest]: ';...
    'Harmonic number of higher harmonic RF system: ';...
    'Higher harmonic RF system amplitude [MV]: ';...
    'Higher harmonic RF system RF Phase [degrees off crest]: ';...
    'Beam Energy [MeV]: ';...
    'Linear Chirp: ';...
    '2nd derivative: ';...
    '3rd derivative: ';...
    'Mirror s? (0/1): ';...
    };
defaultanswer = {
    '0';...
    num2str(nu/1e9);... L_Band
    num2str(a1);... 
    num2str(phi1);...  
    num2str(3);... 3rd harmonic (FLASH and XFEL)
    num2str(a2);...  
    num2str(phi2);... 
    num2str(e0);... 
    num2str(n_der(2));... 
    num2str(n_der(3));... 
    num2str(n_der(4));...
    '0';...
    };
name = 'Edit Longitudinal Phase Space Parameters';
numlines = 1;
options.Resize='on';
options.WindowStyle='normal';
options.Interpreter='tex';
answer = inputdlg(prompt,name,numlines,defaultanswer,options);
if(isempty(answer))
    return
end
%
choice =    str2num(char(answer(1)));
nu =        str2num(char(answer(2)))*1e9;
a1 =        str2num(char(answer(3)));
phi1 =      str2num(char(answer(4)));
n =         str2num(char(answer(5)));
a2 =        str2num(char(answer(6)));
phi2 =      str2num(char(answer(7)));
e0 =        str2num(char(answer(8)));
n_der(2) =  str2num(char(answer(9)));
n_der(3) =  str2num(char(answer(10)));
n_der(4) =  str2num(char(answer(11)));
mirror   =  str2num(char(answer(12)));
%
% depending on choice set up new longitudinal phase space -----------------
switch choice
    case 0
        % change e0 according to new amplitudes
        e0 = a1*cos(phi1/180*pi) + a2*cos(phi2/180*pi);                
    case 1
        n_der = [1; n_der(2:4)];
        % calculate amps and phases from energy and derivatives
        [a1 a2 phi1 phi2] =  knobs_to_amp_phase(e0,nu,n,n_der);
end
        e  = 1/e0 * (a1*cos(2*pi  *nu*(s/c) + phi1/180*pi) + ...
                     a2*cos(2*pi*n*nu*(s/c) + phi2/180*pi)) -1 + de;  
%
% plot --------------------------------------------------------------------
p = polyfit(s,e,3);
plot_long_ps(nu,n,s,e,p,a1,a2,phi1,phi2);             
%
% put into phasespace -----------------------------------------------------
gammag = e0/0.511;
data.ps(data.my_axes).gamma = gammag;
x(:,6) = e;   
data.ps(data.my_axes).phasespace(:,6) = x(:,6);
if(mirror)
    x(:,5) = -x(:,5);
    data.ps(data.my_axes).phasespace(:,5) = x(:,5);
end
%
% re-calculate phase space parameters
[x,b_mean,b_rms,b_emit,b_twiss,weight] = beam_statistics(data.ps(data.my_axes).phasespace,gammag);
%
% put into structure 'ps'
data.ps(data.my_axes).b_stat.x = x;
data.ps(data.my_axes).b_stat.b_mean = b_mean;
data.ps(data.my_axes).b_stat.b_rms = b_rms;
data.ps(data.my_axes).b_stat.b_emit = b_emit;
data.ps(data.my_axes).b_stat.b_twiss = b_twiss;
data.ps(data.my_axes).b_stat.weight = weight;
%
%--- plot
data = psv_plot_ps( data, 0 );
    
return
%
%==========================================================================
%
function [a1 a2 phi1 phi2] =  knobs_to_amp_phase(e0,nu,n,n_der)
% speed of light
c = 3e8;
% wave number
k=2*pi/c * nu;
% rf matrix
Mrf = [ 1       0       1           0;
        0       -k      0           -n*k;
        -k^2    0       -(n*k)^2    0;
        0       k^3     0           (n*k)^3;];
%    
% amplitude/phase vector (a1*cos(phi1) a1*sin(phi1) a2*cos(phi2) a2*sin(phi2)  
Aphi = Mrf^-1 * n_der;
%
a1 = e0 * sqrt( Aphi(1)^2 + Aphi(2)^2 );
a2 = e0 * sqrt( Aphi(3)^2 + Aphi(4)^2 );
phi1 = atan2(Aphi(2),Aphi(1))/pi * 180;
phi2 = atan2(Aphi(4),Aphi(3))/pi * 180;
return
%
%
%==========================================================================
%
function plot_long_ps(nu,n,s,e,p,a1,a2,phi1,phi2)
%
% speed of light
c = 3e8;
%
tagname = 'EDIT_CHIRP';
if findobj('Tag', tagname)
    figure(findobj('Tag', tagname));
else
    figure('Tag','EDIT_CHIRP');
end
plot(s,e,'.');
hold on
plot(s,polyval(p,s),'.r');
%
e0 = a1*cos(phi1/180*pi) + a2*cos(phi2/180*pi);
plot(s,a1/e0*cos(2*pi*nu*(s/c) + phi1/180*pi) + a2/e0*cos(2*pi*n*nu*(s/c) + phi2/180*pi) -1 ,'*g');

title('Longitudinal phase space has been fitted and data will be used as default values')
xlabel('Longitudinal Position')
ylabel('Relative Momentum Offset')
limits = [get(gca,'xlim') get(gca,'ylim')];
thistext1 = ['rf system 1: ' num2str(nu/1e9) ' GHz    rf system 2: ' num2str(n*nu/1e9) ' GHz '];
thistext2 = ['Amplitude 1: ' num2str(a1,5) ' MV  Amplitude 2: ' num2str(a2,5) ' MV'];  
thistext3 = ['Phase 1: ' num2str(phi1,5) ' degree  Phase 2: ' num2str(phi2,5) ' degree'];
xpos = limits(1) + (limits(2) - limits(1))/20;
ypos = limits(3) + (limits(4) - limits(3))/6;
text(xpos,ypos,thistext1)
ypos = limits(3) + (limits(4) - limits(3))/9;
text(xpos,ypos,thistext2)
ypos = limits(3) + (limits(4) - limits(3))/20;
text(xpos,ypos,thistext3)
hold off